<?php

namespace Oro\Bundle\NavigationBundle\DependencyInjection;

use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\HttpKernel\DependencyInjection\Extension;
use Symfony\Component\DependencyInjection\Loader;
use Symfony\Component\Yaml\Yaml;

/**
 * This is the class that loads and manages your bundle configuration
 *
 * To learn more see {@link http://symfony.com/doc/current/cookbook/bundles/extension.html}
 */
class OroNavigationExtension extends Extension
{
    /**
     * {@inheritDoc}
     */
    public function load(array $configs, ContainerBuilder $container)
    {
        $entitiesConfig = array();
        $titlesConfig = array();

        foreach ($container->getParameter('kernel.bundles') as $bundle) {
            $reflection = new \ReflectionClass($bundle);
            if (is_file($file = dirname($reflection->getFilename()).'/Resources/config/navigation.yml')) {
                $bundleConfig = Yaml::parse(realpath($file));

                // merge entity configs
                if (isset($bundleConfig['oro_menu_config'])) {
                    foreach ($bundleConfig['oro_menu_config'] as $entity => $entityConfig) {
                        if (isset($entitiesConfig['oro_menu_config'][$entity])) {
                            $entitiesConfig['oro_menu_config'][$entity] =
                                array_replace_recursive($entitiesConfig['oro_menu_config'][$entity], $entityConfig);
                        } else {
                            $entitiesConfig['oro_menu_config'][$entity] = $entityConfig;
                        }
                    }
                }

                if (isset($bundleConfig['oro_titles'])) {
                    $titlesConfig += is_array($bundleConfig['oro_titles']) ? $bundleConfig['oro_titles'] : array();
                }
            }
        }

        // process configurations to validate and merge
        $configuration = new Configuration();
        $config = $this->processConfiguration($configuration, $entitiesConfig);

        $loader = new Loader\YamlFileLoader($container, new FileLocator(__DIR__.'/../Resources/config'));
        $loader->load('services.yml');
        $container->setParameter('oro_menu_config', $config);
        $container->setParameter('oro_titles', $titlesConfig);

        $container->setParameter('oro_navigation.assets_config', $this->getAssets($container));
    }

    /**
     * Get array with assets from config files
     *
     * @param ContainerBuilder $container
     *
     * @return array
     */
    public function getAssets(ContainerBuilder $container)
    {
        $bundles = $container->getParameter('kernel.bundles');
        $assets = array(
            'css' => array(),
            'js'  => array()
        );

        foreach ($bundles as $bundle) {
            $reflection = new \ReflectionClass($bundle);
            if (is_file($file = dirname($reflection->getFilename()) . '/Resources/config/assets.yml')) {
                $bundleConfig = Yaml::parse(realpath($file));
                if (isset($bundleConfig['css'])) {
                    $assets['css'] = array_merge($assets['css'], $bundleConfig['css']);
                }
                if (isset($bundleConfig['js'])) {
                    $assets['js'] = array_merge($assets['js'], $bundleConfig['js']);
                }
            }
        }

        return $assets;
    }
}
